class Detailer:
    """
    This class is responsible for producing visual representation of the segmentation quality when confronted
    with ground truth.
    """
    def __init__(self, path, regions):
        pass

    def save_image_evaluation(self, data_point_eval):
        pass